/*
 *                          The Art Of
 *                      D E M O M A K I N G
 *
 *
 *                     by Alex J. Champandard
 *                          Base Sixteen
 *
 *                     Modified by Paulo Pinto (Moondevil)
 *                         for using with SDL
 *
 *
 *                http://www.flipcode.com/demomaking
 *
 *                This file is in the public domain.
 *                      Use at your own risk.
 *
 *
 *   Compile with DJGPP:    gcc *.cpp -s -O2 -o stars.exe -lstdcxx
 *
 */

#include <SDL.h>
#include <iostream>
#include <stdlib.h>    // for function rand()
#include "vga.h"

using namespace std;

// change this to adjust the number of stars
#define MAXSTARS 256

// this record contains the information for one star
struct TStar
{
       float x, y;             // position of the star
       unsigned char plane;    // remember which plane it belongs to
};

// this is a pointer to an array of stars
TStar *stars;

// this handles all the graphics
VGA *vga;

// the main program
int main()
{
// Initialize the SDL system
  if (SDL_Init( SDL_INIT_VIDEO ) < 0) {
    cerr << "Couldn't initialize SDL: " << SDL_GetError () << endl;
    exit( -1 );
  }
  atexit( SDL_Quit );

// allocate memory for all our stars
    stars = new TStar[MAXSTARS];
// randomly generate some stars
    for (int i=0; i<MAXSTARS; i++)
    {
        stars[i].x = rand() % 320;
        stars[i].y = rand() % 200;
        stars[i].plane = rand() % 3;     // star colour between 0 and 2
    }
// set mode 320x200x8
    vga = new VGA;
// we need 4 colours shaded from black to white
    vga->SetColour( 0,  0,  0,  0 );  // black (default for index 0 anyway)
    vga->SetColour( 1, 47, 79, 79 );  // dark grey
    vga->SetColour( 2, 211, 211, 211 );  // light grey
    vga->SetColour( 3, 255, 255, 255 );  // white
// run the main loop
    bool done=false;
    while (!done)
    {
      SDL_Event event;
      if (SDL_PollEvent( &event ))
      {
	switch (event.type) {
	case SDL_KEYDOWN: if (event.key.keysym.sym == SDLK_ESCAPE)
	  done = true;
	  break;
	  
	case SDL_QUIT: done = true;
	  break;
	}
      }
	
    // clear the temporary buffer
       vga->Clear();
    // update all stars
       for (int i=0; i<MAXSTARS; i++)
       {
        // move this star right, determine how fast depending on which
        // plane it belongs to
           stars[i].x += (1+(float)stars[i].plane)*0.15;
        // check if it's gone out of the right of the screen
           if (stars[i].x>320)
           {
           // if so, make it return to the left
              stars[i].x = 0;
           // and randomly change the y position
              stars[i].y = rand() % 200;
           }
        // draw this star, with a colour depending on the plane
           vga->PutPixel( (int)stars[i].x, (int)stars[i].y, 1+stars[i].plane );
       }
    // dump our temporary buffer to the screen
       vga->Update();
    }
// return to text mode
    delete (vga);
// free memory
    delete [] (stars);
// make sure every one knows about flipcode :)
    cout << "                             The Art Of" << endl;
    cout << "                         D E M O M A K I N G " << endl << endl;
    cout << "                       by Alex J. Champandard" << endl << endl;
    cout << "       Go to http://www.flipcode.com/demomaking for more information." << endl;
// we've finished!
    return 0;
}
